@@ -5,7 +5,7 @@
 # import the necessary packages
 
 #  ls zen_192.168.0.91_20211105_* | while read line; do echo file \'$line\'; done | ffmpeg -protocol_whitelist file,pipe -f concat -i - -c copy zen_192.168.0.91_20211105_.avi
-
+from flashcam.version import __version__
 from imutils.video import VideoStream
 import socket
 import time
@@ -42,6 +42,12 @@ import math
 
 import requests # crosson=CROSSON
 
+#------------------problem with SHIFT
+from pynput.keyboard import Key, Listener
+
+
+global SHIFT, CTRL
+SHIFT = CTRL = False
 
 global centerX1, centerY1
 global FILE_USERPASS
@@ -104,7 +110,44 @@ HELPTEXT=""" a/A ... SAVE AVI/stop (~/DATA/) (c-a PNG)
 
 
 
+
+
+
+def kb_on_press(key):
+    global SHIFT, CTRL
+    try:
+        a= key.char
+        #print('alphanumeric key {0} pressed'.format( key.char))
+        #CTRL  = False
+    except AttributeError:
+        #print('special key {0} pressed'.format(  key))
+        if key==Key.ctrl:
+            CTRL = True
+        if key==Key.shift:
+            SHIFT = True
+
+def kb_on_release(key):
+    global SHIFT, CTRL
+    if key==Key.ctrl: CTRL = False
+    if key==Key.shift: SHIFT = False
+    #print('{0} released'.format( key))
+    if key == Key.esc:
+        # Stop listener
+        return False
+
+
+
+
+
+
+
+
 def remap_keys(key):
+    """
+    Problem - THIS TABLE WORKS ON  gigajm
+        , while zen : 65505
+    """
+    #print(f" KEY==/{key}/")
     table = {1048673:'a',
              1048674:'b',
              1048675:'c',
@@ -166,11 +209,12 @@ def remap_keys(key):
 #         1310841): #ctrly
 # ctrla
 
+    #####return table[key]
     if key in table:
-        #key = ord( table[key] )
-        keynew = table[key]
-        #print(f"in remaping {key} - {keynew}")
-        return keynew
+        key = ord( table[key] )
+        #keynew = table[key]
+        ##print(f"in remaping {key} - {keynew}")
+        ##return keynew
     return key
 
 
@@ -336,7 +380,9 @@ def display2(videodev, save = False,
     #sname,sfilenamea,sme,sfilenamea,sfilenamea,sfourcc,saviout
 
     sme = socket.gethostname()
-    #frame = None
+    #frame = None  #  gigavg a vaio have strange behaviour
+
+    global CTRL, SHIFT
     global centerX1, centerY1, clicked
     global FILE_USERPASS, FILE_REDCROSS
     global show_help
@@ -348,6 +394,14 @@ def display2(videodev, save = False,
 
     filesource = False
 
+    #==========================  I fire here the listener from pynput !!!
+    listener = Listener(
+        on_press=kb_on_press,
+        on_release=kb_on_release)
+    listener.start()
+    #------------------------------------------------------ let;s see
+
+
     def MOUSE(event, x, y, flags, param):
         global centerX1,centerY1, clicked
         if event == cv2.EVENT_LBUTTONDOWN:
@@ -413,6 +467,9 @@ Recommendation:
         #sfilenamea = f"{sfilenamea}.mp4"
         sfilenamea = f"{sfilenamea}.mkv"
 
+        # XVID-works     LAGS   X264    MP4V? mp4v-new,ubu22    IYUV-huge
+        CODEC = "mp4v"
+
         dir2create = os.path.expanduser("~/DATA/")
         if not os.path.isdir( os.path.expanduser(dir2create )):
             print(f"D... trying to create directory {dir2create} for saving")
@@ -421,21 +478,23 @@ Recommendation:
 
         sfilenamea = os.path.expanduser("~/DATA/" + sfilenamea)
         # codec
-        sfourcc = cv2.VideoWriter_fourcc(*'XVID')
+        #sfourcc = cv2.VideoWriter_fourcc(*'XVID')
         #### sfourcc = cv2.VideoWriter_fourcc(*'LAGS') # lossless NOT mkv,avi
         #sfourcc = cv2.VideoWriter_fourcc(*'X264') # +avi small NOTubu22
-        sfourcc = cv2.VideoWriter_fourcc(*'mp4v') # works on UBU22 clean inst
+        #sfourcc = cv2.VideoWriter_fourcc(*'mp4v') # works on UBU22 clean inst
         #####sfourcc = cv2.VideoWriter_fourcc(*'MP4V')
         #####sfourcc = cv2.VideoWriter_fourcc(*'IYUV') # huge + avi
+
+        sfourcc = cv2.VideoWriter_fourcc(*f'{CODEC}') #
+
         #  25 FPS
         saviout = cv2.VideoWriter( sfilenamea,sfourcc,25.0, resolution )
-        print( sfilenamea )
-        print( sfilenamea )
-        print( sfilenamea )
-        print( sfilenamea )
+        for i in range(4):
+            print( f"SAVE={sfilenamea} ---  FOURCC={CODEC}")
         return sfilenamea,saviout
 
     def get_stream():
+        global CTRL, SHIFT
         # localuser
         global FILE_USERPASS, FILE_REDCROSS
         stream = None # i return
@@ -825,6 +884,9 @@ Recommendation:
 
                 #cv2.namedWindow( wname , cv2.WINDOW_KEEPRATIO ) #
                 # QT cv2.namedWindow( wname , cv2.WINDOW_GUI_NORMAL + cv2.WINDOW_OPENGL  ) #
+                # try this
+                #cv2.namedWindow( wname , cv2.WINDOW_KEEPRATIO ) # 2 may allow resize on gigavg
+                # NEW IN 1.1.8
                 cv2.namedWindow( wname , cv2.WINDOW_GUI_NORMAL   ) #
                 # cv2.namedWindow( wname , 2 ) # 2 may allow resize on gigavg
                 if frames_total < 2:
@@ -1303,10 +1365,22 @@ in progress"""
                 # cv2.namedWindow(rpi_name, cv2.WINDOW_GUI_EXPANDED)
                 # time.sleep(0.2)
                 cv2.setMouseCallback(rpi_name, MOUSE)
-                #key = cv2.waitKey(1) # same problem with shift on ubu22 as ex
-                key = cv2.waitKeyEx(1)
-                #print(f" read {key}")
-                key = remap_keys(key) # make compatible with waitKey()
+                #
+                #  waitkeyex sometimes sees shift and ctrl... but not on zen...
+                #
+                key = cv2.waitKey(1) # same problem with shift on ubu22 as ex
+                #
+                #key = cv2.waitKeyEx(1)
+
+                c = s = "     "
+                if CTRL: c="CTRL "
+                if SHIFT: s="SHIFT"
+                #print(f" {c} x {s} ...      ", end = "       \n")
+                if key!=-1:
+                    print(f" readkey== {key}   {chr(0xFF&key)}    {c} x {s}    >>>. ")
+                    if SHIFT: key = ord(chr(key).upper())
+                # i will not use the translation,,
+                #key = remap_keys(key) # make compatible with waitKey()
                 #print(f"remapped {key}")
 
 
@@ -1361,6 +1435,7 @@ in progress"""
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('q')):
                     print("q PRESSED!")
+                    print("i... it was version:", __version__)
                     sys.exit(1)
 
 
@@ -1934,9 +2009,12 @@ in progress"""
 
 
                 # TEST key code
-                if key!=-1:
-                    print(f"\n{key}\n")
+                #if key!=-1:
+                #    print(f"\n{key}\n")
+
+        print("i... it was version", __version__)
+
 
 if __name__=="__main__":
-    Fire( display3)
+    Fire( display2)
     #Fire({ "disp":display2,   "disp2":display2    })
